@@ -24,8 +24,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 
-__all__ = ('PyMongo', 'ASCENDING', 'DESCENDING', 'PRIMARY',
-           'SECONDARY', 'SECONDARY_ONLY')
+__all__ = ('PyMongo', 'ASCENDING', 'DESCENDING')
 
 from bson.errors import InvalidId
 from bson.objectid import ObjectId
@@ -35,24 +34,25 @@ from mimetypes import guess_type
 from pymongo import uri_parser
 from werkzeug.wsgi import wrap_file
 from werkzeug.routing import BaseConverter
+from pymongo.read_preferences import ReadPreference
 import pymongo
 import warnings
 
 from flask_pymongo.wrappers import MongoClient
 from flask_pymongo.wrappers import MongoReplicaSetClient
 
+import sys
 
+PY2 = sys.version_info[0] == 2
 
-PRIMARY = pymongo.ReadPreference.PRIMARY
-"""Send all queries to the replica set primary, and fail if none exists."""
+# Python 3 compatibility
+if PY2:
+    text_type = (str, unicode)
+    num_type = (int, long)
+else:
+    text_type = str
+    num_type = int
 
-SECONDARY = pymongo.ReadPreference.SECONDARY
-"""Distribute queries among replica set secondaries unless none exist or
-are up, in which case send queries to the primary."""
-
-SECONDARY_ONLY = pymongo.ReadPreference.SECONDARY_ONLY
-"""Distribute queries among replica set secondaries, and fail if none
-exist."""
 
 DESCENDING = pymongo.DESCENDING
 """Descending sort order."""
@@ -60,16 +60,6 @@ DESCENDING = pymongo.DESCENDING
 ASCENDING = pymongo.ASCENDING
 """Ascending sort order."""
 
-READ_PREFERENCE_MAP = {
-    # this handles defaulting to PRIMARY for us
-    None: PRIMARY,
-
-    # alias the string names to the correct constants
-    'PRIMARY': PRIMARY,
-    'SECONDARY': SECONDARY,
-    'SECONDARY_ONLY': SECONDARY_ONLY,
-}
-
 
 class BSONObjectIdConverter(BaseConverter):
     """A simple converter for the RESTful URL routing system of Flask.
@@ -81,10 +71,11 @@ class BSONObjectIdConverter(BaseConverter):
             task = mongo.db.tasks.find_one_or_404(task_id)
             return render_template('task.html', task=task)
 
-    It checks the validate of the id and converts it into a
-    :class:`~bson.objectid.ObjectId` object. The converter will be
-    automatically registered by the initialization of
-    :class:`~flask_pymongo.PyMongo` with keyword :attr:`ObjectId`.
+    Valid object ID strings are converted into
+    :class:`~bson.objectid.ObjectId` objects; invalid strings result
+    in a 404 error. The converter is automatically registered by the
+    initialization of :class:`~flask_pymongo.PyMongo` with keyword
+    :attr:`ObjectId`.
     """
 
     def to_python(self, value):
@@ -146,6 +137,8 @@ class PyMongo(object):
             app.config[key('PASSWORD')] = parsed['password']
             app.config[key('REPLICA_SET')] = parsed['options'].get('replica_set')
             app.config[key('MAX_POOL_SIZE')] = parsed['options'].get('max_pool_size')
+            app.config[key('SOCKET_TIMEOUT_MS')] = parsed['options'].get('socket_timeout_ms', None)
+            app.config[key('CONNECT_TIMEOUT_MS')] = parsed['options'].get('connect_timeout_ms', None)
 
             # we will use the URI for connecting instead of HOST/PORT
             app.config.pop(key('HOST'), None)
@@ -158,6 +151,8 @@ class PyMongo(object):
             app.config.setdefault(key('DBNAME'), app.name)
             app.config.setdefault(key('READ_PREFERENCE'), None)
             app.config.setdefault(key('AUTO_START_REQUEST'), True)
+            app.config.setdefault(key('SOCKET_TIMEOUT_MS'), None)
+            app.config.setdefault(key('CONNECT_TIMEOUT_MS'), None)
 
             # these don't have defaults
             app.config.setdefault(key('USERNAME'), None)
@@ -180,16 +175,24 @@ class PyMongo(object):
             raise Exception('Must set both USERNAME and PASSWORD or neither')
 
         read_preference = app.config[key('READ_PREFERENCE')]
-        read_preference = READ_PREFERENCE_MAP.get(read_preference, read_preference)
-        if read_preference not in (PRIMARY, SECONDARY, SECONDARY_ONLY):
-            raise Exception('"%s_READ_PREFERENCE" must be one of '
-                            'PRIMARY, SECONDARY, SECONDARY_ONLY (was %r)'
-                            % (config_prefix, read_preference))
+        if isinstance(read_preference, text_type):
+            # Assume the string to be the name of the read
+            # preference, and look it up from PyMongo
+            read_preference = getattr(ReadPreference, read_preference)
+            if read_preference is None:
+                raise ValueError(
+                    '%s_READ_PREFERENCE: No such read preference name (%r)' % (
+                        config_prefix, read_pref))
+            app.config[key('READ_PREFERENCE')] = read_preference
+        # Else assume read_preference is already a valid constant
+        # from pymongo.read_preferences.ReadPreference or None
 
         replica_set = app.config[key('REPLICA_SET')]
         dbname = app.config[key('DBNAME')]
         auto_start_request = app.config[key('AUTO_START_REQUEST')]
         max_pool_size = app.config[key('MAX_POOL_SIZE')]
+        socket_timeout_ms = app.config[key('SOCKET_TIMEOUT_MS')]
+        connect_timeout_ms = app.config[key('CONNECT_TIMEOUT_MS')]
 
         # document class is not supported by URI, using setdefault in all cases
         document_class = app.config.setdefault(key('DOCUMENT_CLASS'), None)
@@ -199,11 +202,18 @@ class PyMongo(object):
 
         args = [host]
         kwargs = {
-            'read_preference': read_preference,
+            'auto_start_request': auto_start_request,
             'tz_aware': True,
         }
 
-        kwargs['auto_start_request'] = auto_start_request
+        if read_preference is not None:
+            kwargs['read_preference'] = read_preference,
+
+        if socket_timeout_ms is not None:
+            kwargs['socketTimeoutMS'] = socket_timeout_ms
+
+        if connect_timeout_ms is not None:
+            kwargs['connectTimeoutMS'] = connect_timeout_ms
 
         if replica_set is not None:
             kwargs['replicaSet'] = replica_set
@@ -268,11 +278,11 @@ class PyMongo(object):
         :param int cache_for: number of seconds that browsers should be
            instructed to cache responses
         """
-        if not isinstance(base, (str, unicode)):
+        if not isinstance(base, text_type):
             raise TypeError('"base" must be string or unicode')
-        if not isinstance(version, (int, long)):
+        if not isinstance(version, num_type):
             raise TypeError('"version" must be an integer')
-        if not isinstance(cache_for, (int, long)):
+        if not isinstance(cache_for, num_type):
             raise TypeError('"cache_for" must be an integer')
 
         storage = GridFS(self.db, base)
@@ -317,7 +327,7 @@ class PyMongo(object):
            ``None``, the content-type is guessed from the filename using
            :func:`~mimetypes.guess_type`
         """
-        if not isinstance(base, (str, unicode)):
+        if not isinstance(base, text_type):
             raise TypeError('"base" must be string or unicode')
         if not (hasattr(fileobj, 'read') and callable(fileobj.read)):
             raise TypeError('"fileobj" must have read() method')
